// ライセンス: GPL2

//#define _DEBUG
#include "jddebug.h"

#include "imageareabase.h"

#include "jdlib/miscmsg.h"
#include "jdlib/miscgtk.h"

#include "dbimg/imginterface.h"
#include "dbimg/img.h"

#include "httpcode.h"

using namespace IMAGE;


ImageAreaBase::ImageAreaBase( const std::string& url )
    : m_url( url ),
      m_img ( DBIMG::get_img( m_url ) ),
      m_ready( false ),
      m_width( 0 ),
      m_height( 0 )
{
    assert( m_img );

    set_alignment( Gtk::ALIGN_CENTER, Gtk::ALIGN_CENTER );
}



ImageAreaBase::~ImageAreaBase()
{
#ifdef _DEBUG
    std::cout << "ImageAreaBase::~ImageArea url = " << m_url << std::endl;
#endif 
}


//
// 画像表示
//
void ImageAreaBase::set_image()
{
    // 画像ロード
    bool stop = false;
    std::string errmsg;

    Glib::RefPtr< Gdk::PixbufLoader > loader = MISC::get_ImageLoder( m_img->get_cache_path(), get_width(), get_height(), stop, errmsg );
    if( loader ){

        if( m_img->get_mosaic() ) set_mosaic( loader->get_pixbuf() );
        else set( loader->get_animation() );
    }
    else{
        set_errmsg( errmsg );
        MISC::ERRMSG( get_errmsg() );
    }

    set_ready( true );
}


//
// モザイク表示
//
void ImageAreaBase::set_mosaic( Glib::RefPtr< Gdk::Pixbuf > pixbuf )
{
    int size_mosaic = 20;  // モザイク画像は 1/size_mosaic にしてもとのサイズに直す
    if( get_width() / size_mosaic < 16 ) size_mosaic = MAX( 1, get_width() / 16 );

    Glib::RefPtr< Gdk::Pixbuf > pixbuf2;
    pixbuf2 = pixbuf->scale_simple( get_width() / size_mosaic, get_height() / size_mosaic, Gdk::INTERP_NEAREST );
    set( pixbuf2->scale_simple( m_width, m_height, Gdk::INTERP_NEAREST ) );
}
